@@ -6,8 +6,9 @@ use warnings;
 use base 'RxPerl::Subject';
 
 use Carp 'croak';
+use Scalar::Util 'weaken';
 
-our $VERSION = "v6.1.1";
+our $VERSION = "v6.1.2";
 
 sub _on_subscribe {
     my ($self, $subscriber) = @_;
@@ -24,15 +25,14 @@ sub new {
 
     $self->{_last_value} = $initial_value;
 
+    weaken(my $w_self = $self);
+    my $next_orig = $self->{next};
+    $self->{next} = sub {
+        $w_self->{_last_value} = $_[0] unless $self->{_closed};
+        $next_orig->(@_);
+    };
+
     bless $self, $class;
 }
 
-sub next {
-    my $self = shift;
-
-    $self->{_last_value} = $_[0] unless $self->{_closed};
-
-    $self->SUPER::next(@_);
-}
-
 1;
